import importlib
from typing import Callable
from typing import List
from typing import Tuple

from . import hparms
from . import color
from .parm_types import HouParmType
from ceonstock import CstockJobInput, CstockJobInputType


# To prevent flake8 from failing on the string type-hints during CI
if False:
    import hou

importlib.reload(hparms)


def hou_parm_type(job_input_type: CstockJobInputType):
    return CSTOCK_TO_HOU[job_input_type]


def create_file_input_parm(parm_name: str, value: str):
    value_parm = hparms.create_file_parm(
        parm_name,
        label=parm_name,
        default_value=f"$CSTOCK/job_inputs/{value}",
    )
    return value_parm


def create_color_input_parm(parm_name: str, hex_value):
    value = list(color.color_hex_to_hou(hex_value))
    value_parm = hparms.create_color_parm(
        parm_name,
        label=parm_name,
        # default_value=[1, 0.2, 0.5],
        default_value=value,
    )
    return value_parm


def create_toggle_input_parm(parm_name: str, bool_value: bool):
    value = bool_value
    value_parm = hparms.create_toggle_parm(
        parm_name,
        label=parm_name,
        # default_value=[1, 0.2, 0.5],
        default_value=value,
    )
    return value_parm


def create_string_input_parm(parm_name, value):
    # TODO parse hex string to produce value
    value_parm = hparms.create_text_parm(
        parm_name, label=parm_name, default_value=value
    )
    return value_parm


# Map a function responsible for generating the parms for each cstock
# job input type. The function is expected to receive a name and
# a value, and returns a single hou.Parm.
CSTOCK_TO_HOU = {
    CstockJobInputType.IMG: create_file_input_parm,
    CstockJobInputType.STRING: create_string_input_parm,
    CstockJobInputType.COLOR: create_color_input_parm,
    CstockJobInputType.BOOL: create_toggle_input_parm,
}


def get_value_parm_fn(
    cstock_job_input_type: CstockJobInputType,
) -> Callable[[str, str], "hou.Parm"]:
    found_fn = CSTOCK_TO_HOU[cstock_job_input_type]
    return found_fn


def job_input_parms(cstock_job_input: CstockJobInput):
    """
    Receive a CstockJobInput instance.
    Returns a list of parms for the CstockJobInput instance.
    """
    print(f"Creating parms for job_input: {cstock_job_input}")
    new_parms = []  # List of hou parm instances
    job_input_name = cstock_job_input.name
    input_type = cstock_job_input.job_input_type
    # TODO setup 'entries' to store a value + metadata (e.g. img AR)
    entries = cstock_job_input.values or []

    num_entries_parm_name = f"{job_input_name}_num_entries"
    num_entries_parm = hparms.create_int_parm(
        num_entries_parm_name,
        label="Num Entries",
        default_value=len(entries),
    )
    new_parms.append(num_entries_parm)

    ###
    # Create a value parm for each entry in the job_input.
    ###
    # 'Value parm' is the parm that stores the received value for a particular
    # cstock_job_input entry.
    parm_fn = get_value_parm_fn(input_type)
    # TODO create 'entries' that store a value along with associated metadata.
    for i, value in enumerate(cstock_job_input.values or []):
        parm_name = f"{job_input_name}_{i}"
        value_parm = parm_fn(parm_name, value)
        new_parms.append(value_parm)

    return new_parms



# Functions that help with the generation of project inputs.
import logging
from typing import List, Dict

from ceonstock import CstockJobInputType, CstockProjectInput
from ceonstock import CstockJobInput
from ceonstock import CstockSubmissionInput
from ceonstock import CstockWebInputType
from ceonstock.core.submission_input import CstockSubmissionInputEntry
from ceonstock.core.submission_input import CstockSubmissionInputRaw
from ceonstock.core.file_action import (
    CstockFileActionCrop,
    CstockFileActionDocToImgs,
)
from ceonstock.core.file_action import CstockFileAction
from ceonstock.core.file_action import CstockFileActionType
from ceonstock.log import printify

logger = logging.getLogger(__name__)

PREPROCESS_ACTIONS: Dict[CstockWebInputType, List[CstockFileAction]] = {
    CstockWebInputType.DOC: [
        CstockFileAction(
            CstockFileActionType.DOC_TO_IMGS, CstockFileActionDocToImgs()
        )
    ]
}


def create_submission_input_raw(
    cstock_project_input: CstockProjectInput,
    values: List,
) -> CstockSubmissionInputRaw:
    # TODO validate values are valid for this project input?
    # Get required actions for preprocessing this input.
    preprocess_actions = PREPROCESS_ACTIONS.get(
        cstock_project_input.web_input_type, []
    )
    submission_input_raw = CstockSubmissionInputRaw(
        values=values,
        project_input=cstock_project_input,
        preprocess_actions=preprocess_actions,
    )
    return submission_input_raw


def create_submission_input(
    cstock_project_input: CstockProjectInput,
    values: List,
) -> CstockSubmissionInput:
    """Receives a project input and the provided user entries.
    Returns a submission input."""
    entries = [
        CstockSubmissionInputEntry(value=value, entry_actions=[])
        for value in values
    ]
    return CstockSubmissionInput(
        project_input=cstock_project_input,
        entries=entries,
    )


# def create_submission(
#     project_inputs: List[CstockProjectInput], user_inputs_raw: List[CstockSubmissionInputRaw]
# ):
#     """Build the submission input instances from the received raw json from the web client."""
#     project_inputs_lookup = {
#         project_input.name: project_input for project_input in project_inputs
#     }

#     submission_inputs = []
#     for user_input_raw in user_inputs_raw:
#         entries = [SubmissionInputEntry(value=value, entry_actions=[]) for value in user_input_raw.values]
#         submission_input = CstockSubmissionInput(entries=entries, project_input=cstock_project_input)
#         submission_inputs.append(submission_input)
#     logger.info(f"Create submission inputs:{printify(submission_inputs)}")


def raw_to_submission_input(
    submission_input_raw: CstockSubmissionInputRaw,
) -> CstockSubmissionInput:
    """
    Convert raw input to submission input.
    Assumes that no pre-processing is required.
    In the case that pre-processign is required, it is the responsibility of the preprocessor
    to provide valid submission_inputs after preprocessing has completed.
    """
    if len(submission_input_raw.preprocess_actions) >= 1:
        # Don't allow this to be used if preprocessing is required
        raise Exception(
            f"Failed to convert raw submission inputs: "
            f"\t{submission_input_raw.project_input.name} contains preprocessing tasks: {submission_input_raw.preprocess_actions}"
            f"In this case it is the responsibility of the preprocessing server to generate correct submission inputs."
        )

    # Create an entry for each value
    entries = [
        CstockSubmissionInputEntry(value=value, entry_actions=[])
        for value in submission_input_raw.values
    ]
    return CstockSubmissionInput(
        entries=entries, project_input=submission_input_raw.project_input
    )


def raw_to_submission_inputs(
    user_inputs_raw: List[CstockSubmissionInputRaw],
) -> List[CstockSubmissionInput]:
    """
    Connvert a list of CstockSubmissionInputRaw to CstockSubmissionInput.
    """
    return [
        raw_to_submission_input(input_raw) for input_raw in user_inputs_raw
    ]


def submission_input_to_job_input(
    submission_input: CstockSubmissionInput,
) -> CstockJobInput:
    values = [entry.value for entry in submission_input.entries]
    return CstockJobInput(
        name=submission_input.project_input.name,
        job_input_type=submission_input.project_input.job_input_type,
        values=values,
    )


# TODO remove in favor of single conversion
# Currently kepts only for backwards compatibility.
def submission_inputs_to_job_inputs(
    submission_inputs: List[CstockSubmissionInput],
) -> List[CstockJobInput]:
    job_inputs = [
        submission_input_to_job_input(input) for input in submission_inputs
    ]
    return job_inputs


def validate_submission(
    project_inputs: List[CstockProjectInput],
    submission_inputs: List[CstockSubmissionInput],
):
    # TODO validate e.g. submission is missing required project inputs, or includes an unrecognized input..
    raise NotImplementedError
    pass


def submission_input_expects_files(
    submission_input: CstockSubmissionInput,
) -> bool:
    """Returns True if the submission receives files from the user.
    Returns False if no files are expected."""
    if (
        submission_input.project_input.web_input_type
        == CstockWebInputType.DROPDOWN
    ):
        # Although a dropdown web_input may produce a job_input that is a file (E.g. an img file), it does
        # not receive the file from the customer (In this case the image would be a fixed path in the project files)
        return False
    job_input_filetypes = [CstockJobInputType.IMG, CstockJobInputType.AUDIO]
    if submission_input.project_input.job_input_type in job_input_filetypes:
        return True
    return False



import logging
import inspect
from typing import Type, TypeVar
from ceonstock import errors

T = TypeVar("T")

logger = logging.getLogger(__name__)


def cstock_from_dict(dict_kwargs: dict, cls_to_instantiate: Type[T]) -> T:
    """
    Init from a dict, but fail and reporting if unrecognized kwargs are found.
    -
    This was added because instantiating with cls(**kwargs_dict) silently ignores unrecognized kwargs
    when running in FastAPI (FastAPI overrides dataclasses default behaviour with pydantic versions???).
    This meant that projects which were identified as valid in the webUI/FileServer would fail when trying to run
    on the render server (if unrecognized kwargs were found).
    Therefore, use this function to instantiate classes from a dict.
    """
    unrecognized_kwargs = []
    known_kwargs = {}
    for k, v in dict_kwargs.items():
        if k in inspect.signature(cls_to_instantiate).parameters:
            known_kwargs[k] = v
        else:
            unrecognized_kwargs.append(k)

    if unrecognized_kwargs:
        logger.warning(
            f"Received {len(unrecognized_kwargs)} unrecognized kwargs: {unrecognized_kwargs}"
        )
        raise errors.CstockInstantiationError(
            cstock_cls=cls_to_instantiate,
            message=f"Unrecognized kwargs: {unrecognized_kwargs}",
        )

    return cls_to_instantiate(**known_kwargs)



from dataclasses import dataclass

from .base import CstockBaseEnum, CstockBaseClass


class CstockFileSourceType(CstockBaseEnum):
    PROJECT = "project"
    JOB_INPUT = "job_input"
    JOB_OUTPUT = "job_output"
    # TASK_INPUT = "task_input"
    TASK_OUTPUT = "task_output"
    ABSOLUTE = "absolute"


@dataclass
class CstockFileReference(CstockBaseClass):
    target: str
    file_source: CstockFileSourceType = CstockFileSourceType.PROJECT



import logging
from ceonstock.log import formats
from enum import Enum

from .colors import TerminalColor


class ColoredFormatter(logging.Formatter):
    default_format = (
        "%(asctime)s - %(name)s - %(levelname)s - %(message)s (%(filename)s:%(lineno)d)"
    )

    # def __init__(self, *args, **kwargs):
    #     super().__init__(*args, **kwargs)

    def __init__(self, string_for_format: str, *args, **kwargs):
        self.msg_format = ColoredFormatter.default_format
        if string_for_format:
            self.msg_format = string_for_format
        # super().__init__(*args, **kwargs)

    def get_color(self, levelno):
        COLORS = {
            logging.DEBUG: TerminalColor.BLUE,
            logging.INFO: TerminalColor.GREY,
            logging.WARNING: TerminalColor.YELLOW,
            logging.ERROR: TerminalColor.RED,
            logging.CRITICAL: TerminalColor.RED_BOLD,
        }
        chosen_color = COLORS.get(levelno, "")
        return chosen_color

    def apply_color(self, formatted_msg: str, terminal_color: TerminalColor):
        # The color formatting must be applied to each line individually to take effect.
        colored_lines = []
        for line in formatted_msg.split("\n"):
            colored_line = f"{terminal_color.value}{line}{TerminalColor._RESET.value}"
            colored_lines.append(colored_line)
        colored_msg = "\n".join(colored_lines)
        return colored_msg

    def format(self, record):
        log_fmt = self.msg_format
        chosen_color = self.get_color(record.levelno)
        formatter = logging.Formatter(log_fmt)
        formatted_msg = formatter.format(record)
        colored_msg = self.apply_color(formatted_msg, chosen_color)
        return colored_msg


DEV = ColoredFormatter(formats.DEV)
# PROD = ColoredFormatter(formats.PROD)
PROD = logging.Formatter(formats.PROD)



import customtkinter
from project_input import FrameProjectInput
from typing import List
from typing import Optional
from pathlib import Path

import ceonstock
from ceonstock import CstockProjectInput
from ceonstock import CstockProject
from ceonstock import CstockRenderPipeline
from ceonstock import json_io

DEFAULT_JOB_INPUT_TYPE = CstockProjectInput.job_input_types.IMG
DEFAULT_WEB_INPUT_TYPE = CstockProjectInput.web_input_types.IMG


class FrameProjectInputs(customtkinter.CTkScrollableFrame):
    def __init__(self, master):
        super().__init__(master)
        self.grid_columnconfigure(0, weight=1)
        # self.project_inputs: List[CstockProjectInput] = []
        # self.project_input_frames: List[FrameProjectInput] = []

    def add_project_input(self):
        print("Adding new poroject input")
        new_inputs = self.get()
        new_inputs.append(
            # TODO allow CstockProjectInput to be initialized without web_input_type.
            # Move logic for getting valid web_input_types to cstock module.
            CstockProjectInput(
                name="new_proj_input",
                job_input_type=DEFAULT_JOB_INPUT_TYPE,
                web_input_type=DEFAULT_WEB_INPUT_TYPE,
                description="Describe the project input",
            )
        )
        self.update_project_inputs(new_inputs)

    # def remove_project_input(self):
    #     print("Removing last poroject input")
    #     new_inputs = self.project_inputs[:-1]
    #     self.update_project_inputs(new_inputs)
    #     self.project_inputs = new_inputs

    def get(self) -> List[CstockProjectInput]:
        print("Getting project inputs from UI ...")
        return [frame.get() for frame in self.winfo_children()]

    def update_project_inputs(
        self, new_project_inputs: List[CstockProjectInput]
    ):
        print("Updating UI")
        # Fetch the new list
        # TODO get current inputs from widgets/frames procedurally.
        # Do not store cstock_objects in the variable.
        original_inputs = self.get()
        new_inputs = new_project_inputs

        # Store the length of the current list and the new list
        cur_len, new_len = len(original_inputs), len(new_inputs)
        print(f"{cur_len=}")
        print(f"{new_len=}")

        # If the length of new list is more than current list then
        if new_len > cur_len:
            diff = new_len - cur_len

            # Change text of existing widgets
            for idx, wid in enumerate(self.winfo_children()):
                # wid.config(text=new_items[idx])
                print("TODO apply change")

            # Make the rest of the widgets required
            for i in range(diff):
                print("Adding new pojrect_input widget set.")
                # Button(items_frame, text=new_items[cur_len+i]).pack()
                row = cur_len + i
                cstock_project_input = new_inputs[row]
                # print(f"{row=}")
                new_frame = FrameProjectInput(self, cstock_project_input)
                new_frame.grid(row=row, column=0, pady=(0, 20), sticky="ew")
                # self.project_input_frames.append(new_frame)

        # If the length of current list is more than new list then
        elif new_len < cur_len:
            extra = cur_len - new_len

            # Change the text for the existing widgets
            for idx in range(new_len):
                wid = self.winfo_children()[idx]
                print("TODO apply changes on existing")
                # wid.config(text=new_items[idx])

            # Get the extra widgets that need to be removed
            extra_wids = [
                wid for wid in self.winfo_children()[-1 : -extra - 1 : -1]
            ]  # The indexing is a way to pick the last 'n' items from a list

            # Remove the extra widgets
            for wid in extra_wids:
                wid.destroy()

            # Also can shorten the last 2 steps into a single line using
            # [wid.destroy() for wid in items_frame.winfo_children()[-1:-extra-1:-1]]

        # self.project_inputs = (
        #     new_inputs  # Update the value of the main list to be the new list
        # )


class App(customtkinter.CTk):
    def __init__(self):
        super().__init__()

        self.version = ceonstock.__version__
        print(self.version)
        self.title(f"CeonStock Project Maker - v{self.version}")
        self.geometry("800x620")
        self.grid_columnconfigure((0, 1), weight=1)
        self.grid_rowconfigure(0, weight=1)

        self.frame_project_inputs = FrameProjectInputs(self)
        self.frame_project_inputs.grid(
            row=0, column=0, padx=10, pady=10, sticky="nsew", columnspan=2
        )

        # Add button
        self.button_add_input = customtkinter.CTkButton(
            self,
            text="+ Project Input",
            command=self.frame_project_inputs.add_project_input,
        )
        self.button_add_input.grid(
            row=1, column=1, padx=10, pady=10, sticky="ew"
        )

        # Save/Export
        self.button_export = customtkinter.CTkButton(
            self, text="Export", command=self.export_callback
        )
        self.button_export.grid(
            row=2, column=0, padx=10, pady=10, sticky="ew", columnspan=2
        )

    def export_callback(self):
        print("Button Clicked!")
        project_inputs = self.frame_project_inputs.get()
        print(project_inputs)
        file = Path("./ceonstock_project_test.json")
        print("Making file: ", file)
        print(f"Project inputs ({len(project_inputs)}): ")
        render_pipeline = CstockRenderPipeline([], "")
        project = CstockProject(
            project_inputs=project_inputs, render_pipeline=render_pipeline
        )
        for project_input in project_inputs:
            print(project_input)
        # json_io.file_handler.write_json_file(
        #     {"project_inputs": project_inputs}, file
        # )
        json_io.file_handler.write_json_file(project.__dict__, file)
        # with open(file, "w") as f:
        #     f.write("testdata")


# TODO bug:
# After deleting, then creating a new input, the number of inputs is incorrect.
if __name__ == "__main__":
    app = App()
    print(
        "WARNING: TODO FIx bug when deleting and then creating a new input (leaves overwritten input as existing)"
    )
    app.mainloop()



from .core import CstockWebInputType
from .core import CstockJob
from .core import CstockJobType
from .core import CstockJobInput
from .core import CstockJobInputType

from .core.project import CstockProject
from .core.project import CstockProjectInfo

# Added for backward compatability.
# TODO zone this out in favor of CsotckProjectInput
from .core import CstockProjectInput as CstockProjInput
from .core import CstockProjectInput

# from .core import CstockRenderPipeline
# from .core import CstockRenderTask
# from .core import CstockRenderAppType

from .core import CstockSubmissionInput

from . import json_io
from . import software_integrations

from .version import __version__



from dataclasses import dataclass
from typing import List, Dict, Optional
from copy import deepcopy

from ..core import proj_input_type as cprojtype
from ..core import job_input_type as cjobtype
from ..core.proj_input import CstockProjInputGroup
from ..log import create_new_logger
from ..utils import printify
from ..core.base import ICstockPipelineData
from ..version import VERSION
from .actions import SubmissionAction

logger = create_new_logger(__name__)

# TODO use these? NOt currently used


@dataclass
class CstockSubmissionInputEntry(ICstockPipelineData):
    value: str
    actions: List[SubmissionAction]

    @classmethod
    def from_dict(cls, dict_data):
        logger.debug(f"<{cls.__name__}>.from_dict got dict_data: {printify(dict_data)}")
        args = {
            "value": dict_data["value"],
            "actions": [
                SubmissionAction.from_dict(process_action)
                for process_action in dict_data["actions"]
            ],
        }
        logger.debug(f"<{cls.__name__}>.from_dict created args: {args}")
        return cls(**args)

    def to_dict(self):
        new_dict = {
            "value": self.value,
            "actions": [action.to_dict() for action in self.actions],
        }
        return new_dict


@dataclass
class CstockSubmissionInput:
    """The submission entries related to a single CstockProjInput"""

    name: str
    entries: List[CstockSubmissionInputEntry]
    proj_input_type: cprojtype.CstockProjInputType

    @classmethod
    def from_dict(cls, dict_data):
        args = {
            "name": dict_data["name"],
            "entries": [
                CstockSubmissionInputEntry.from_dict(submission_entry)
                for submission_entry in dict_data["entries"]
            ],
        }
        if dict_data.get("proj_input_type"):
            args["proj_input_type"] = cprojtype.CstockProjInputType(
                dict_data["proj_input_type"]
            )
        return cls(**args)

    def to_dict(self):
        new_dict = {
            "name": self.name,
            "entries": [
                submission_entry.to_dict() for submission_entry in self.entries
            ],
        }
        if self.proj_input_type:
            new_dict["proj_input_type"] = self.proj_input_type.value
        return new_dict

    @property  # for parity with access style of proj_input_type
    def job_input_type(self) -> cjobtype.CstockJobInputType:
        # TODO refactor job_input_type to use a type_info class, same format as proj_input_type
        return self.proj_input_type.info().job_input_type

    def values(self):
        return [entry.value for entry in self.entries]

    def remove_actions(self):
        for entry in self.entries:
            entry.actions = []


class CstockSubmissionInputGroup(ICstockPipelineData):
    """Works with a set of SubmissionInputs"""

    def __init__(self, submission_inputs: List[CstockSubmissionInput]):
        self._submission_inputs = submission_inputs

    def __str__(self):
        msg = f"<{type(self).__name__}>"
        return msg

    @classmethod
    def from_raw_entries(
        cls,
        proj_input_group: CstockProjInputGroup,
        user_entries_raw: Dict[str, List[str]],
    ):
        submission_inputs_list = []
        for proj_input in proj_input_group.proj_inputs():
            default_preprocess_actions = proj_input.type().info().preprocess_actions
            logger.debug(f"default_preproces_actions: {default_preprocess_actions}")
            current_input_entries = user_entries_raw[proj_input.name]
            logger.debug(f"current_input_entries: {current_input_entries}")
            user_input_entries = [
                CstockSubmissionInputEntry(entry, default_preprocess_actions)
                for entry in current_input_entries
            ]
            logger.debug(f"Created user_input_entries: {user_input_entries}")
            user_input = CstockSubmissionInput(
                proj_input.name, user_input_entries, proj_input.proj_input_type
            )
            logger.debug(f"Created user_input: {user_input}")
            submission_inputs_list.append(user_input)
        return cls(submission_inputs_list)

    @classmethod
    def from_dict(cls, dict_data):
        submission_inputs = [
            CstockSubmissionInput.from_dict(submission_input)
            for submission_input in dict_data["submission_inputs"]
        ]
        return cls(submission_inputs)

    def to_dict(self):
        """Returns a serializable dict"""
        submission_inputs = dict()
        submission_inputs["submission_inputs"] = [
            submission_input.to_dict() for submission_input in self._submission_inputs
        ]
        submission_inputs["ceonstock_version"] = VERSION

        return submission_inputs

    def submission_inputs(self) -> List[CstockSubmissionInput]:
        return self._submission_inputs



import logging
from ceonstock import CstockProject
from ceonstock import CstockProjectInput
from ceonstock import CstockJob
from ceonstock import CstockJobInputType
from ceonstock import CstockWebInputType
from ceonstock.core.project import CstockProjectInfo
from ceonstock.core.project import CstockRenderPipeline
from ceonstock.log import printify

from ceonstock.job_maker import project_preview_video_maker
from ceonstock.job_maker import job_maker

from ceonstock.log import formatters, formats

root_logger = logging.getLogger()
root_logger.setLevel("INFO")

formatter = formatters.ColoredFormatter(formats.DEV)
sh = logging.StreamHandler()
sh.setFormatter(formatter)

root_logger.addHandler(sh)

logger = logging.getLogger(__name__)

PROJECT_INPUT_PREVIEW_FILES_DIR_DEFAULT = "/mnt/FileStorage/Dayne/Web/proj_ceonstock/assets/project_input_files/project_preview_video_inputs/default_inputs"
PROJECT_INPUT_PREVIEW_FILES_DIR_EXAMPLES = "/mnt/FileStorage/Dayne/Web/proj_ceonstock/assets/project_input_files/project_preview_video_inputs/example_inputs"
JOBS_DIR = "/mnt/FileStorage/Dayne/Web/proj_ceonstock/local_storage/jobs"

# PROJECT_INPUTS = [
#     CstockProjectInput(
#         name="pen_color",
#         job_input_type=CstockJobInputType.COLOR,
#         web_input_type=CstockWebInputType.COLOR,
#         description="Color of pen",
#     ),
#     CstockProjectInput(
#         name="flag_image",
#         job_input_type=CstockJobInputType.IMG,
#         web_input_type=CstockWebInputType.IMG,
#         description="Image on flag",
#     ),
#     CstockProjectInput(
#         name="box_color",
#         job_input_type=CstockJobInputType.COLOR,
#         web_input_type=CstockWebInputType.COLOR,
#         description="Color of box",
#     ),
# ]
PROJECT_INPUTS = [
    CstockProjectInput(
        name="single_image",
        job_input_type=CstockJobInputType.IMG,
        web_input_type=CstockProjectInput.web_input_types.IMG,
        description="A single image TODO system to enforce limit on num inputs",
    ),
    CstockProjectInput(
        name="image_from_doc",
        job_input_type=CstockJobInputType.IMG,
        web_input_type=CstockProjectInput.web_input_types.DOC,
        description="A single image TODO system to enforce limit on num inputs",
    ),
    CstockProjectInput(
        name="single_color",
        job_input_type=CstockJobInputType.COLOR,
        web_input_type=CstockProjectInput.web_input_types.COLOR,
        description="A single color",
    ),
    CstockProjectInput(
        name="dropdown_string",
        job_input_type=CstockJobInputType.STRING,
        web_input_type=CstockProjectInput.web_input_types.DROPDOWN,
        type_settings={
            "dropdown_options": [
                {
                    "value": "dropdown_option_value_0",
                    "label": "Option Label 0",
                },
                {
                    "value": "dropdown_option_value_1",
                    "label": "Option Label 1",
                },
                {
                    "value": "dropdown_option_value_2",
                    "label": "Option Label 2",
                },
            ]
        },
        description="A single image TODO system to enforce limit on num inputs",
    ),
]

RENDER_PIPELINE = CstockRenderPipeline(
    render_tasks=[], primary_output="out", extra_outputs=[]
)

PROJECT_INFO = CstockProjectInfo(
    "My Test Project",
    description="Testing the genration of jobs for hte preview video",
    tags=[],
)

PROJECT_TO_TEST = CstockProject(
    project_inputs=PROJECT_INPUTS,
    render_pipeline=RENDER_PIPELINE,
    project_info=PROJECT_INFO,
)


def main(ceonstock_project: CstockProject):
    project_input_jobs: dict[
        str, list[CstockJob]
    ] = project_preview_video_maker.create_jobs_for_project_preview_video(
        ceonstock_project,
        files_dir_default=PROJECT_INPUT_PREVIEW_FILES_DIR_DEFAULT,
        files_dir_examples=PROJECT_INPUT_PREVIEW_FILES_DIR_EXAMPLES,
    )
    print_header("Created project_input jobs:")
    logger.info(f"{printify(project_input_jobs)}")

    # Test execute
    print()
    project_input_name_to_test: str = list(project_input_jobs.keys())[1]
    print_header(
        f"Testing with isolated project input: {project_input_name_to_test}"
    )
    jobs_for_project_input = project_input_jobs[project_input_name_to_test]
    file_sources = [
        PROJECT_INPUT_PREVIEW_FILES_DIR_DEFAULT,
        PROJECT_INPUT_PREVIEW_FILES_DIR_EXAMPLES,
    ]
    for job in jobs_for_project_input:
        logger.info(f"{job}")
        job_maker.create_cstock_job_on_disk(job, JOBS_DIR, file_sources)


def print_header(msg):
    logger.info(f"*****\n{msg}\n*****")


if __name__ == "__main__":
    main(PROJECT_TO_TEST)



# A helper class that loads a target dir as a CstockProj
# Useful for validating the integrity of the project files
# Useful for identifying the file path that represents the project in a server with a filesystem
import logging
from typing import Union
from pathlib import Path

from ceonstock import CstockProject
from ceonstock import json_io

logger = logging.getLogger(__name__)
# logger.setLevel("DEBUG")


class CstockLocalProject:
    """
    Represents a folder in the local filesystem that contains a cstock project.
    Provides utility to load cstock clases from the local files.
    """

    def __init__(
        self,
        project_folder_path: Union[Path, str],
        json_file_name: str = "ceonstock_project.json",
    ):
        self._root = Path(project_folder_path)
        self._json_file_path = Path(self._root, json_file_name)
        if not self._root.is_dir():
            raise NotADirectoryError(
                f"Local_project dir does not exist: {self._root}"
            )
        if not self._json_file_path.is_file():
            raise FileNotFoundError(
                f"Local_project {self._root} missing expected JSON file: {self._json_file_path.name}"
            )

    def path(self) -> Path:
        return self._root

    def json_path(self) -> Path:
        return self._json_file_path

    def load(self) -> CstockProject:
        """
        Loads a CstockProject instance from the json file.
        Loading imports all data as python/ceonstock classes
        """
        logger.debug("\n\n")
        cstock_project = json_io.project.from_file(str(self.json_path()))
        logger.debug(
            f"Loaded CstockProject instance: {self.json_path()}\n{cstock_project}"
        )
        return cstock_project

    def __str__(self):
        return f"<{self.__class__.__name__} {self.path()}>"

    def __repr__(self):
        return f"<{self.__class__.__name__} {self.path()}>"
